package Plugins::Search;

#======================================================================

#----------------------------------------------------------------------
#  .
#
use strict;
use Setup qw/$CONFIG $VARS/;
use CH::Getform;
use CH::DBUtil;
use vars qw/@ISA/;

@ISA = qw/Board/;

#----------------------------------------------------------------------
#
#
sub new {
	bless my $self = {};
	
	$self->{TEMPLATE} = DB()->load_template("search_result.html");
	$self->{TEMPLATE}->param(DB->style_vars(" ˻"));
	$self->_init(@_);
	$self->{TEMPLATE}->output;
}

#----------------------------------------------------------------------
#
#
sub _init {
	my $self = shift;
	my (%threads, %topic, $no_match);
	my ($threads_total, $hits, $count) = 0;
	my (@search_forum, @search_field, @forum_list) = ();
	my $data = []; 
	
	@search_field = $self->search_field($IN->{field});
	
	$IN->{query} =~ s/^\s*//g;
	$IN->{query} =~ s/\s*$//g;

	my $query = $IN->{query};
	
	if ($IN->{forum} eq "all") {
		my $cat = DB->get_data($CONFIG->{cat_file});
		foreach (keys %$cat) {
			foreach (@{$cat->{$_}->{ForumList}}) {
				push @search_forum, $_;
			}
		}
	}
	else {
		@search_forum = $IN->{forum};
	}

	foreach my $forum (@search_forum) {
		$data = DB->read_file("$CONFIG->{forum_path}/$forum/topic.db");
		LINE: foreach (@$data) {
			my @values = DB->data_split($_);
			
			if ( $values[1] < ( DB->sort_time() - $IN->{date}) and $IN->{date} ne "all" ) {
				next LINE;
			}
			else {
				push @{$topic{$forum}},  shift @values;
			}
		}
	}
	
	if ($IN->{field} eq "all") {
		@search_field = ( 2, 3, 4 );
	}

	foreach my $forum (keys %topic) {
		foreach my $t (@{$topic{$forum}}) {
			open (FH, "<$CONFIG->{forum_path}/$forum/$t.cgi");
				flock(FH, 1);
				my $file_found = 0;
				
				LINE: while(<FH>) {
					next LINE if /^\s*$/;
					next LINE if /^TOPIC/;
					chomp;
			
					my @values = DB->data_split($_);
					$threads_total++;
					my $found = 0;
			
					FIELD: foreach my $field (@search_field) {
						next FIELD if (length $query < 2);
						$_ = $values[$field];
						$found = m,\Q$query\E,io;
						$found and last FIELD;
					}
					if ($found) {
						$file_found = 1;
						$hits++;
					}
				}

			push @{$threads{$forum}}, $t if $file_found;
			close FH;
		}
	}
	
	if (keys %threads > 0) {
		my ($userid, $cat, $forumtype, $foruminfo, %list_row, $insert, $id, $forum, $msg);
		my $data_row = [];
		my $html_output = '';
		
		foreach $forum (sort keys %threads) {
			$cat = $self->_category($forum);
			$foruminfo = DB->forum_data($forum);
			$forumtype = $self->forum_type($foruminfo->{Type});
			
			$html_output .= qq|<tr><td colspan="7" $VARS->{forum_bgcolor}>
			<img src="$CONFIG->{images_url}/folder.gif" align="absmiddle">
			<$VARS->{font}>
			<a href="index.cgi?ch=Forum&do=PostList&cid=$cat&b=$forum">
			<b>$foruminfo->{Name}($forumtype) </a></b></font>
			</td></tr>|;
			
			foreach $id (@{$threads{$forum}}) {
				%list_row;
				$insert = DB->insert_data($forum, $id);

				while ( my($k, $v) = each(%$insert)) {
					$list_row{$k} = $v;
				}
				
				%list_row = $self->search_result($cat, $forum, $id, %list_row);
				
				$list_row{BGCOLOR} = DB->color($count);
				$list_row{NUMBER} = $count + 1;
				$list_row{NUMBER} = "<$VARS->{font}>$list_row{NUMBER}</font>";
				
				$html_output .= DB->insert_tpl(\%list_row, "threads_list.html");
				$count++;
			}
		}
		$msg = qq|<$VARS->{font}>ã ˻ <b>$IN->{query}</b>  <b>$hits</b> .</font>|;

		$self->{TEMPLATE}->param( 
					result 	=> $html_output,
					found 	=> 1,
					hits	=> $msg
				);
	} 
	elsif ($query eq '') { 
		$self->{TEMPLATE}->param( 
				no_match 	=> "˻ Է ʾҽϴ.",
				);
	}
	else { 
		$self->{TEMPLATE}->param( 
				no_match 	=> "˻ <B>$query</B>() ã Ͽϴ.",
				);
	}
	
	my $hidden_tag = qq|
	<input type="hidden" name="ch" value="Forum">
	<input type="hidden" name="do" value="Search">
	|;

	$self->{TEMPLATE}->param ( 
				forum_list	=> $self->_forumlist($IN->{forum}),
				hidden_tag 	=> $hidden_tag,
				term		=> $query,
			);

}

#----------------------------------------------------------------------
#
#
sub search_result {
	my ($self, $cat, $forum, $id, %list_row) = @_;
	my ($toolbar, $attach_file, $open_close, $t_page);

	$attach_file = ($list_row{ATTACH}) ? 
	"<A HREF=\"index.cgi?ch=Forum&do=AttachFile&ID=ID=$forum,$id&f=$list_row{ATTACH}\"><IMG SRC=\"$CONFIG->{images_url}/disk.gif\" BORDER=\"0\" ALT=\"$list_row{ATTACH}, $list_row{ATTACH_TYPE}, $list_row{ATTACH_SIZE} Ʈ\" ALIGN=\"ABSMIDDLE\"></A>" : 0;
			
	$list_row{SUBJECT} = DB->trim_text($list_row{SUBJECT}, $CONFIG->{subject_length});
	$list_row{SUBJECT} = "<$VARS->{font}><a href=\"index.cgi?ch=Forum&do=ReadPost&cid=$cat&b=$forum&term=$IN->{query}&t=$id\">$list_row{SUBJECT}</a></font>";
	$list_row{SUBJECT} .= "&nbsp;$attach_file" if $attach_file;

	#----------------------------------------------------------------------
	# 亯 .
	#----------------------------------------------------------------------
			
	if ( $list_row{REPLIES} > $CONFIG->{threads_perpage} ) {
		$t_page  = int($list_row{REPLIES} / $CONFIG->{threads_perpage}) + 1;
		for (1..$t_page) {
		$toolbar .= "<a href=\"index.cgi?ch=Forum&do=ReadPost&cid=$cat&b=$forum&t=$id&page=$_\">$_</a> ";
		}
		$toolbar .= "[<a href=\"index.cgi?ch=Forum&do=ReadPost&cid=$cat&b=$forum&t=$id&page=all\">κ</a>]";
		$list_row{SUBJECT} .= "&nbsp;: <img src=\"$CONFIG->{images_url}/posticon.gif\" width=\"12\" height=\"14\" align=\"absmiddle\">&nbsp;$toolbar";
	}
			
	#----------------------------------------------------------------------
	# hot icon .
	#----------------------------------------------------------------------

	if ($list_row{REPLIES} >= $CONFIG->{hot_icon}) {
		$list_row{SUBJECT} .= "&nbsp;<IMG SRC=\"$CONFIG->{images_url}/hot.gif\" align=\"absmiddle\">";
	}

	#----------------------------------------------------------------------
	# ȸ  ȸ ũŰ ڿ    .
	#----------------------------------------------------------------------
			
	if ($list_row{USER_ID} ne "guest") {
		my $poster = $self->member_data($list_row{USER_ID});
		my $userid = $poster->{USER_ID};
		$list_row{AUTHOR} = "<$VARS->{font}><a href=index.cgi?ch=Member&do=member_profile&ID=$userid>$list_row{AUTHOR}</a></font>";
		if ($poster->{MEMBER_LEVEL} == (9 || 3)) {
			$list_row{AUTHOR} .= $self->member_icon($poster->{MEMBER_LEVEL});
		}
	}
	else { 
		$list_row{AUTHOR} = "<$VARS->{font}>$list_row{AUTHOR}</font>";
	}
			
	$open_close = "open";
	
	if ($list_row{CLOSE}) {
		$open_close = "locked";
	}
	
	$list_row{file_icon} = "<img src=\"$CONFIG->{images_url}/$open_close.gif\"><BR>";
	
	return %list_row;
}

#----------------------------------------------------------------------
#
#
sub _category {
	my ($self, $forum) = @_;
	my $cat = DB->get_data($CONFIG->{cat_file});
	my $r_data;
	
	LINE: foreach my $conf (keys %$cat) {
		foreach (@{$cat->{$conf}->{ForumList}}) {
			$forum eq $_ and $r_data = $conf and last LINE;
		}
	}
	return $r_data;
}

#----------------------------------------------------------------------
#
#		
sub _forumlist {
	my ($self, $select_forum) = @_;
	my $list = DB->get_data($CONFIG->{cat_file});
	my ($forum, $forum_list);
	my $nomatch = 1;
	
	$forum_list = "<select name=forum>\n";
	foreach (keys %$list) {
		foreach (@{$list->{$_}->{ForumList}}) {
			$forum = DB->forum_data($_);
			$forum_list .= "<option value=$_";
			if ($select_forum eq $_) {
				$forum_list .= " selected";
				$nomatch = 0;
			}
			$forum_list .= ">$forum->{Name} (" 
					. $self->forum_type($forum->{Type}) 
					. ")\n";
		}
	}

	$forum_list .= "<option value=all";
	$forum_list .= " selected" if $nomatch;
	$forum_list .= "> ";
	$forum_list .= "</select>";
	
	return $forum_list;
}

#----------------------------------------------------------------------
#
#
sub search_field {
	my $self = shift;
	my $field = shift;
	
	my %search_field = (
		subject	=> 2,
		body	=> 3,
		name	=> 4,
	);
	return $search_field{$field};
}

#----------------------------------------------------------------------
